local lsputil = require('lspconfig.util')
local servers_path = vim.fn.stdpath('data') .. '/lsp'

-- Read a file line by line and build a table from them
---@param file string
local function read_dict(file)
    if not file then
        return nil
    end
    local dict = {}
    for line in io.lines(file) do
        table.insert(dict, line)
    end
    return dict
end

local function get_binary_paths(binaries)
    local paths = {}
    for _, bin in ipairs(binaries) do
        local path = vim.fn.exepath(bin)
        if path ~= '' then
            table.insert(paths, path)
        end
    end
    return table.concat(paths, ',')
end

local function deno_root_dir(fname)
    return lsputil.root_pattern('deno.json', 'deno.jsonc')(fname)
end

return {
    ansiblels = { cmd = { servers_path .. '/ansiblels/node_modules/.bin/ansible-language-server', '--stdio' } },

    bashls = { cmd = { servers_path .. '/bashls/node_modules/.bin/bash-language-server', 'start' } },

    beancount = { init_options = { journalFile = os.getenv('HOME') .. '/Documents/Ledger/ledger.beancount' } },

    biome = {},

    clangd = {
        filetypes = { 'c', 'cpp', 'objc', 'objcpp', 'cuda' },
        keys = {
            { '<localleader>h', '<cmd>ClangdSwitchSourceHeader<CR>', mode = 'n', desc = 'Switch source header' },
        },
        init_options = {
            clangdFileStatus = true,
        },
        cmd = {
            'clangd',
            '-j=4',
            '--enable-config',
            '--query-driver=' .. get_binary_paths { 'clang', 'clang++', 'gcc', 'g++' },
            '--background-index',
            '--clang-tidy',
            '--all-scopes-completion',
            '--completion-style=detailed',
            '--pch-storage=memory',
            '--header-insertion=iwyu',
            '--header-insertion-decorators',
            '--import-insertions',
        },
    },

    cssls = { cmd = { servers_path .. '/vscode/node_modules/.bin/vscode-css-language-server', '--stdio' } },

    -- Ref: https://docs.deno.com/runtime/manual/advanced/language_server/overview
    denols = {
        -- Ensure `tsserver` and `denols` aren't attached to a single buffer at the same time
        root_dir = deno_root_dir,
        init_options = { enable = true, lint = true },
        settings = {
            deno = {
                unstable = true,
                -- codeLens = {
                --     implementations = true,
                --     references = true,
                --     referencesAllFunctions = true,
                -- },
                suggest = {
                    imports = {
                        autoDiscover = false,
                        hosts = {
                            ['https://deno.land'] = false,
                            ['https://nest.land'] = false,
                        },
                    },
                },
            },
        },
    },

    dhall_lsp_server = {},

    dockerls = { cmd = { servers_path .. '/dockerls/node_modules/.bin/docker-langserver', '--stdio' } },

    emmet_language_server = {
        cmd = { servers_path .. '/emmet_language_server/node_modules/.bin/emmet-language-server', '--stdio' },
    },

    gleam = {},

    -- NOTE: for Bazel setup (GOPACKAGESDRIVER), use neoconf.nvim
    -- Ref: https://github.com/bazelbuild/rules_go/wiki/Editor-setup
    gopls = {
        cmd = { 'gopls', '-remote=auto', 'serve' },
        settings = {
            gopls = {
                gofumpt = true,
                codelenses = {
                    gc_details = false,
                    generate = true,
                    regenerate_cgo = true,
                    run_govulncheck = true,
                    tidy = true,
                    upgrade_dependency = true,
                    vendor = true,
                },
                hints = {
                    assignVariableTypes = true,
                    compositeLiteralFields = true,
                    compositeLiteralTypes = true,
                    constantValues = true,
                    functionTypeParameters = true,
                    parameterNames = true,
                    rangeVariableTypes = true,
                },
                analyses = {
                    fieldalignment = true,
                    nilness = true,
                    shadow = true,
                    unusedparams = true,
                    unusedvariable = true,
                    unusedwrite = true,
                    useany = true,
                },
                usePlaceholders = true,
                staticcheck = true,
                -- semanticTokens = true,
                vulncheck = 'Imports',
                directoryFilters = {
                    '-.git',
                    '-.vscode',
                    '-.vscode-test',
                    '-.idea',
                    '-node_modules',
                    '-plz-out',
                    '-.plz-cache',
                    '-.plz-http-cache',
                    '-bazel-bin',
                    '-bazel-out',
                    '-bazel-testlogs',
                },
            },
        },
    },

    graphql = { cmd = { servers_path .. '/graphql/node_modules/.bin/graphql-lsp', 'server', '-m', 'stream' } },

    guile_ls = {
        filetypes = { 'scheme' },
        root_dir = lsputil.root_pattern('guix.scm', 'manifest.scm', '.guix-channel', '.git'),
    },

    html = { cmd = { servers_path .. '/vscode/node_modules/.bin/vscode-html-language-server', '--stdio' } },

    jinja_lsp = {},

    -- jsonls = {
    --     cmd = { servers_path .. '/vscode/node_modules/.bin/vscode-json-language-server', '--stdio' },
    --     -- Schema catalog: https://www.schemastore.org/api/json/catalog.json
    --     settings = {
    --         json = {
    --             schemas = require('schemastore').json.schemas(),
    --             validate = { enable = true },
    --         },
    --     },
    -- },

    jsonnet_ls = {},

    koka = {},

    ltex = {
        settings = {
            ltex = {
                dictionary = {
                    ['en-US'] = read_dict(vim.fn.stdpath('config') .. '/spell/en.utf-8.add'),
                },
                completionEnabled = true,
            },
        },
    },

    lua_ls = {
        settings = {
            Lua = {
                -- codeLens = { enable = true },
                format = { enable = false },
                hint = { enable = true, arrayIndex = 'Disable', setType = true },
                completion = { callSnippet = 'Replace', keywordSnippet = 'Replace' },
                telemetry = { enable = false },
                workspace = { checkThirdParty = false },
            },
        },
    },

    marksman = {},

    mdx_analyzer = {
        cmd = { servers_path .. '/mdx_analyzer/node_modules/.bin/mdx-language-server', '--stdio' },
        root_dir = lsputil.root_pattern('package.json', 'tsconfig.json'),
    },

    nickel_ls = { single_file_support = true },

    nil_ls = {
        root_dir = lsputil.root_pattern('.git', 'flake.lock'),
        settings = {
            ['nil'] = {
                formatting = { command = { 'nixpkgs-fmt' } },
                -- nix = { flake = { autoEvalInputs = true } },
            },
        },
    },

    nushell = {},

    -- TODO: implement handlers for LSP extensions
    -- Ref: https://github.com/ocaml/ocaml-lsp#lsp-extensions
    ocamllsp = {
        root_dir = lsputil.root_pattern('*.opam', '.ocamlformat', 'esy.json', 'dune-project', 'dune-workspace', '.git'),
    },

    pbls = {},

    -- please = { cmd = { 'build_langserver', '--mode', 'stdio' } },

    purescriptls = {
        cmd = { servers_path .. '/purescriptls/node_modules/.bin/purescript-language-server', '--stdio' },
        -- Ref: https://github.com/nwolverson/vscode-ide-purescript/blob/master/package.json
        settings = {
            purescript = {
                -- addNpmPath = true,
                formatter = 'purs-tidy',
            },
        },
    },

    -- TODO: switch to pylyzer when it is more stable
    pylsp = {
        cmd = { servers_path .. '/pylsp/bin/pylsp' },
        settings = {
            pylsp = {
                plugins = {
                    autopep8 = { enabled = false },
                    pydocstyle = { enabled = true },
                    pylint = { enabled = true },
                },
            },
        },
    },

    -- pylyzer = {},

    qmlls = {}, -- from qt6-qtdeclarative-dev package in Alpine

    regal = {},

    -- regols = {},

    remark_ls = {
        cmd = { servers_path .. '/remark_ls/node_modules/.bin/remark-language-server', '--stdio' },
        single_file_support = false,
    },

    rescriptls = {
        cmd = { 'node', servers_path .. '/rescriptls/server/out/cli.js', '--stdio' },
        init_options = {
            extensionConfiguration = {
                allowBuiltInFormatter = true,
                askToStartBuild = false,
                codeLens = true,
                inlayHints = { enable = true },
            },
        },
    },

    ruff_lsp = { cmd = { servers_path .. '/ruff_lsp/bin/ruff-lsp' } },

    -- use neoconf on each project to override `cmd` to `sqls -config sqls.yml`.
    sqls = { root_dir = lsputil.root_pattern('sqls.yml') },

    svelte = { cmd = { servers_path .. '/svelte/node_modules/.bin/svelteserver', '--stdio' } },

    -- taplo = {},

    templ = {},

    terraformls = {},

    texlab = {
        keys = {
            { '<localleader>p', '<cmd>TexlabForward<CR>', mode = 'n', desc = 'Preview LaTEX buffer' },
        },
        settings = {
            texlab = {
                build = { onSave = true },
                forwardSearch = {
                    executable = 'zathura',
                    args = { '--synctex-forward', '%l:1:%f', '%p' },
                },
            },
        },
    },

    -- tflint = { root_dir = lsputil.root_pattern('.tflint.hcl') },

    tsserver = {
        cmd = { servers_path .. '/tsserver/node_modules/.bin/typescript-language-server', '--stdio' },
        root_dir = function(fname)
            if deno_root_dir(fname) == nil then
                return lsputil.root_pattern('package.json', 'tsconfig.json', 'jsconfig.json', '.git')(fname)
            end
        end,
        single_file_support = false,
    },

    typst_lsp = {},

    volar = {
        cmd = { servers_path .. '/volar/node_modules/.bin/vue-language-server', '--stdio' },
        on_new_config = function(config, root_dir)
            local function get_typescript_server_path(dir)
                local global_ts = vim.fn.stdpath('data') .. '/lsp/volar/node_modules/typescript/lib'
                local found_ts = ''

                if
                    lsputil.search_ancestors(dir, function(path)
                        found_ts = lsputil.path.join(path, 'node_modules', 'typescript', 'lib')
                        if lsputil.path.exists(found_ts) then
                            return path
                        end
                    end)
                then
                    return found_ts
                end
                return global_ts
            end

            config.init_options.typescript.tsdk = get_typescript_server_path(root_dir)
        end,
    },

    -- yamlls = {
    --     cmd = { servers_path .. '/yamlls/node_modules/.bin/yaml-language-server', '--stdio' },
    --     settings = {
    --         redhat = { telemetry = { enabled = false } },
    --         yaml = {
    --             format = {
    --                 enable = true,
    --                 bracketSpacing = true,
    --                 printWidth = 120,
    --                 singleQuote = true,
    --             },
    --             validate = true,
    --             schemas = require('schemastore').yaml.schemas(),
    --             schemaStore = { enable = false, url = '' },
    --             -- https://docs.gitlab.com/ee/ci/yaml/yaml_optimization.html#configure-your-ide-to-support-reference-tags
    --             customTags = { '!reference sequence' },
    --         },
    --     },
    -- },

    zls = {
        settings = {
            warn_style = true,
            highlight_global_var_declarations = true,
        },
    },
}
