package main

import (
	"context"
	"fmt"
	"os"
	"os/exec"
	"path/filepath"
	"strings"

	"github.com/sirupsen/logrus"
	"golang.org/x/sync/errgroup"
	"mvdan.cc/sh/v3/expand"
	"mvdan.cc/sh/v3/interp"
	"mvdan.cc/sh/v3/syntax"
)

func installPkgGroup(data_dir string, targets map[string]Target, t []string) error {
	errGroup := new(errgroup.Group)
	for _, target := range t {
		target := target
		errGroup.Go(func() error {
			return installPkg(data_dir, targets, target)
		})
	}
	return errGroup.Wait()
}

func installPkg(data_dir string, targets map[string]Target, target string) error {
	// Verify that the target exists
	spec, ok := targets[target]
	if !ok {
		return fmt.Errorf("invalid target '%s'", target)
	}

	targetLog := logrus.WithField("target", target)
	targetLog.Warn("Installing target")

	// Install targets into categorized directories
	install_dir := filepath.Join(data_dir, spec.Category[0], target)

	// Create the parent directory to install packages into (in the case of `python` or `nodejs`).
	// For `custom` type, the script should make this directory in its own way
	if spec.Type == "nodejs" || spec.Type == "python" {
		err := os.MkdirAll(install_dir, 0o755)
		if err != nil {
			return err
		}

		// Use the target's name as its package name, if not specified
		if len(spec.Pkgs) == 0 {
			spec.Pkgs = append(spec.Pkgs, target)
		}
	}

	switch spec.Type {
	case "nodejs":
		for _, pkg := range spec.Pkgs {
			cmd := [3]string{"pnpm", "add", fmt.Sprintf("%s@latest", pkg)}
			targetLog.Infof("%s %s %s", cmd[0], cmd[1], cmd[2])

			execCmd := exec.Command(cmd[0], cmd[1:]...)
			execCmd.Dir = install_dir
			execCmd.Stdout = os.Stdout
			execCmd.Stderr = os.Stderr
			if err := execCmd.Run(); err != nil {
				return fmt.Errorf("failed to install %s: %v", pkg, err)
			}
		}

	case "python":
		// Always upgrade pip along the way
		spec.Pkgs = append(spec.Pkgs, "pip")

		// Create a venv directory to install the target's packages into
		execCmd := exec.Command("python3", "-m", "venv", install_dir)
		execCmd.Stdout = os.Stdout
		execCmd.Stderr = os.Stderr
		if err := execCmd.Run(); err != nil {
			return fmt.Errorf("failed to create python venv: %v", err)
		}

		for _, pkg := range spec.Pkgs {
			cmd := [4]string{filepath.Join("bin", "pip3"), "install", "-U", pkg}
			targetLog.Infof("%s %s %s %s", cmd[0], cmd[1], cmd[2], cmd[3])

			execCmd = exec.Command(cmd[0], cmd[1:]...)
			execCmd.Dir = install_dir
			execCmd.Stdout = os.Stdout
			execCmd.Stderr = os.Stderr
			if err := execCmd.Run(); err != nil {
				return fmt.Errorf("failed to install %s: %v", pkg, err)
			}
		}

	case "custom":
		// Always export `PKG_INSTALL_DIR` for convenient usage in script block.
		// Also inherit from the shell env, to have proper PATH and so on.
		envs := append(os.Environ(), "PKG_INSTALL_DIR="+install_dir)
		for k, v := range spec.Env {
			envs = append(envs, fmt.Sprintf("%s=%s", k, v))
		}

		runner, err := interp.New(
			interp.Env(expand.ListEnviron(envs...)),
			interp.StdIO(nil, os.Stdout, os.Stderr),
		)
		if err != nil {
			return err
		}

		ctx := context.TODO() // just create a dummy context for each custom target
		for _, line := range spec.Script {
			targetLog.Info(line)
			script, err := syntax.NewParser().Parse(strings.NewReader(line), "")
			if err != nil {
				return err
			}
			err = runner.Run(ctx, script)
			if err != nil {
				return err
			}
		}
	}

	return nil
}
