package main

import (
	"os"

	"github.com/sirupsen/logrus"
	"github.com/spf13/cobra"
)

func init() {
	logrus.SetFormatter(&logrus.TextFormatter{
		DisableTimestamp: true,
		DisableQuote:     true,
	})
	logrus.SetOutput(os.Stderr)
	logrus.SetLevel(logrus.InfoLevel)
}

func main() {
	var list, lsp, dap, lint bool

	rootCmd := &cobra.Command{
		Use:           "pkg-install",
		Short:         "A small CLI program to install LSP/DAP servers and various linters/formatters",
		SilenceUsage:  true,
		SilenceErrors: true,
		RunE: func(cmd *cobra.Command, args []string) error {
			// Just print help output if there is nothing to do
			if cmd.Flags().NFlag() == 0 && len(args) == 0 {
				return cmd.Help()
			}

			targets, err := readTargets()
			if err != nil {
				return err
			}

			if list {
				return listPkgs(targets)
			}

			nvim_data_dir, err := findNvimDataDir()
			if err != nil {
				return err
			}

			// We can install multiple categories at the same time
			if lsp {
				logrus.Warn("Installing LSP servers")
				err := installPkgGroup(nvim_data_dir, targets, filterTargets(targets, "lsp"))
				if err != nil {
					return err
				}
			}
			if dap {
				logrus.Warn("Installing DAP servers")
				err := installPkgGroup(nvim_data_dir, targets, filterTargets(targets, "dap"))
				if err != nil {
					return err
				}
			}
			if lint {
				logrus.Warn("Installing linters and formatters")
				err := installPkgGroup(nvim_data_dir, targets, filterTargets(targets, "lint"))
				if err != nil {
					return err
				}
			}

			// Only install specified targets when no categories are selected
			if cmd.Flags().NFlag() == 0 {
				err := installPkgGroup(nvim_data_dir, targets, args)
				if err != nil {
					return err
				}
			}

			return nil
		},
	}

	rootCmd.PersistentFlags().BoolVar(&list, "list", false, "List installable targets")
	rootCmd.PersistentFlags().BoolVar(&lsp, "lsp", false, "Install all LSP servers")
	rootCmd.PersistentFlags().BoolVar(&dap, "dap", false, "Install all DAP servers")
	rootCmd.PersistentFlags().BoolVar(&lint, "lint", false, "Install all linters and formatters")

	err := rootCmd.Execute()
	if err != nil {
		logrus.Error(err)
		os.Exit(1)
	}
}
