package main

import (
	"fmt"
	"os"
	"path/filepath"
	"slices"

	"github.com/go-playground/validator/v10"
	"github.com/goccy/go-yaml"
	"github.com/neovim/go-client/nvim"
)

type Target struct {
	Env      map[string]string `yaml:"env,omitempty"    validate:"excluded_unless=Type custom,omitempty,min=1,dive,keys,min=1,endkeys,required"`
	Type     string            `yaml:"type"             validate:"required,oneof=python nodejs custom"`
	Category []string          `yaml:"category"         validate:"required,min=1,dive,oneof=lsp dap lint"`
	Pkgs     []string          `yaml:"pkgs,omitempty"   validate:"excluded_if=Type custom,omitempty,min=1,dive,min=1"`
	Script   []string          `yaml:"script,omitempty" validate:"required_if=Type custom,omitempty,min=1,dive,min=1"`
}

var validate = validator.New()

func readTargets() (map[string]Target, error) {
	var nvim_config_dir string
	vim, err := nvim.NewChildProcess(nvim.ChildProcessArgs("--embed"))
	if err != nil {
		return nil, err
	}
	if err = vim.ExecLua("return vim.fn.stdpath(...)", &nvim_config_dir, "config"); err != nil {
		return nil, err
	}
	if err = vim.Close(); err != nil {
		return nil, err
	}

	input, err := os.ReadFile(filepath.Join(nvim_config_dir, "pkgs.yaml"))
	if err != nil {
		return nil, err
	}

	targets := map[string]Target{}
	err = yaml.Unmarshal(input, &targets)
	if err != nil {
		return nil, err
	}

	// Validate the received input before return it
	for target, spec := range targets {
		err = validate.Struct(spec)
		if err != nil {
			return nil, fmt.Errorf("[%s] invalid input: %v", target, err)
		}
	}

	return targets, nil
}

func filterTargets(targets map[string]Target, category string) []string {
	result := []string{}
	for name, spec := range targets {
		if slices.Contains(spec.Category, category) {
			result = append(result, name)
		}
	}
	return result
}
